local timer = require("timer")
local screen = require("shack")

local Shadows = require("shadows")
local LightWorld = require("shadows.LightWorld")
local newLightWorld = LightWorld:new()

local Light = require("shadows.Light")

local allEnemies
local numEnemies = 0

function addEnemy(x, y)
    enemy = {}
    enemy.isHit = false
    enemy.body = love.physics.newBody(world, x, y, "dynamic")
    enemy.shape = love.physics.newRectangleShape(50, 100)
    enemy.fixture = love.physics.newFixture(enemy.body, enemy.shape)
    enemy.body:setFixedRotation(true)
    enemy.fixture:setUserData("Enemy")

    table.insert(allEnemies, enemy)
end

function love.load()
    allEnemies = {}
    score = 0

    windowWidth = love.graphics.getWidth()
    windowHeight = love.graphics.getHeight()

    local newLight = Light:new(newLightWorld, windowWidth-200)
    newLight:SetPosition(windowWidth/2, windowHeight/2)
    newLight:SetColor(90, 50, 50)

    mainSong = love.audio.newSource("song.mp3", "stream")
    mainSong:play()
    mainSong:setLooping(true)

    s = love.audio.newSource("explosion.mp3", "static")

    love.physics.setMeter(64)
    world = love.physics.newWorld(0, 9.81*64, true)
    world:setCallbacks(beginContact)

    player = {}
    player.isHit = false
    player.canMove = true
    player.jumps = 1
    player.body = love.physics.newBody(world, windowWidth/2, 700, "dynamic")
    player.shape = love.physics.newRectangleShape(50, 100)
    player.fixture = love.physics.newFixture(player.body, player.shape)
    player.body:setFixedRotation(true)
    player.fixture:setRestitution(0.25)
    player.fixture:setUserData("Player")

    ground = {}
    ground.body = love.physics.newBody(world, windowWidth/2, windowHeight-25, "static")
    ground.shape = love.physics.newRectangleShape(windowWidth, 50)
    ground.fixture = love.physics.newFixture(ground.body, ground.shape)
    ground.fixture:setUserData("Ground")

    rightWall = {}
    rightWall.body = love.physics.newBody(world, windowWidth, windowHeight/2, "static")
    rightWall.shape = love.physics.newRectangleShape(50, windowHeight)
    rightWall.fixture = love.physics.newFixture(rightWall.body, rightWall.shape)
    rightWall.fixture:setUserData("RightWall")

    leftWall = {}
    leftWall.body = love.physics.newBody(world, 0, windowHeight/2, "static")
    leftWall.shape = love.physics.newRectangleShape(50, windowHeight)
    leftWall.fixture = love.physics.newFixture(leftWall.body, leftWall.shape)
    leftWall.fixture:setUserData("LeftWall")

    roof = {}
    roof.body = love.physics.newBody(world, windowWidth/2, 150, "static")
    roof.shape = love.physics.newRectangleShape(windowWidth+200, 50)
    roof.fixture = love.physics.newFixture(roof.body, roof.shape)
    roof.fixture:setUserData("Roof")
    roof.body:setAngle(math.rad(10))

    love.graphics.setBackgroundColor(0.5, 0.5, 0.5, 1)

end


function love.update(dt)
    screen:update(dt)
    world:update(dt)
    timer.update(dt)
    newLightWorld:Update()

    velX, velY = player.body:getLinearVelocity()

    if player.canMove then
        if love.keyboard.isDown("right") then
            player.body:applyForce(400, 0)
        elseif love.keyboard.isDown("left") then
            player.body:applyForce(-400, 0)
        end
    end

    if player.isHit then
        player.body:setAngle(math.rad(90))
        player.canMove = false
    end

end

function love.draw()
    screen:apply()
    love.graphics.setColor(1, 1, 1, 1)
    love.graphics.polygon("fill", player.body:getWorldPoints(player.shape:getPoints()))

    love.graphics.polygon("fill", ground.body:getWorldPoints(ground.shape:getPoints()))
    love.graphics.polygon("fill", rightWall.body:getWorldPoints(rightWall.shape:getPoints()))
    love.graphics.polygon("fill", leftWall.body:getWorldPoints(leftWall.shape:getPoints()))
    love.graphics.polygon("fill", roof.body:getWorldPoints(roof.shape:getPoints()))

    love.graphics.setColor(1, 0, 0, 1)
    for _, enemy in ipairs(allEnemies) do
        if enemy.body:isDestroyed() == false then
            love.graphics.polygon("fill", enemy.body:getWorldPoints(enemy.shape:getPoints()))
        end
    end
    newLightWorld:Draw()
    love.graphics.print("SCORE: "..score, 10, 10)
end

function love.keypressed(key)
    if player.canMove then
        if key == "up" and player.jumps > 0 then
            player.body:applyLinearImpulse(0, -600)
            player.jumps = player.jumps - 1
        end
    end

    if key == "down" then
        player.body:applyLinearImpulse(0, 2500)
    end
end

function beginContact(a, b, coll)
    if a:getUserData() == "Player" and b:getUserData() == "Ground" or a:getUserData() == "Ground" and b:getUserData() == "Player" then
        player.jumps = 1
    end

    for _, enemy in ipairs(allEnemies) do
        if a:getUserData() == "Player" and b == enemy.fixture and velY >= 1000 or a == enemy.fixture and b:getUserData() == "Player" and velY >= 1000 then
            player.jumps = 1
            enemy.isHit = true
            enemy.body:destroy()
            numEnemies = numEnemies - 1
            screen:shake(30)
            score = score + 100
            s:play()
        end
    end

    if a:getUserData() == "Player" and b:getUserData() == "Enemy" and velY <= 1000 or a:getUserData() == "Enemy" and b:getUserData() == "Player" and velY <= 1000 then
        player.isHit = true
    end
end

timer.every(1, function()
    if numEnemies < 10 then
        addEnemy(love.math.random(0, windowWidth), 700)
        numEnemies = numEnemies + 1
    end
end)